-- SSOd

CREATE TABLE schema_version (
    version NUMBER
);

INSERT INTO schema_version VALUES (1);

CREATE TABLE applications (
    id TEXT PRIMARY KEY, -- UUID
    name TEXT NOT NULL,
    display_name TEXT NOT NULL,
    version TEXT NOT NULL,
    capabilities TEXT NOT NULL,
    cert TEXT,
    cert_old TEXT,
    restore_password_url TEXT,
    change_password_url TEXT,
    url TEXT
);

CREATE TABLE users (
    id TEXT PRIMARY KEY, -- UUID
    login TEXT UNIQUE,
    password TEXT NOT NULL,
    password_hash_type TEXT NOT NULL,
    owner_id TEXT NOT NULL
);

CREATE TABLE users_sessions (
    id TEXT PRIMARY KEY,
    user_id TEXT NOT NULL,
    last_access NUMBER NOT NULL,
    expiration_period NUMBER NOT NULL
);

CREATE TABLE users_domain_sessions (
    id TEXT PRIMARY KEY,
    session_id TEXT NOT NULL,
    domain TEXT NOT NULL,
    UNIQUE (session_id, domain)
);

-- users.owner_id foreign key

CREATE TRIGGER fki_users_owner_id
BEFORE INSERT ON users
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((SELECT id FROM applications WHERE id = NEW.owner_id) IS NULL)
    THEN RAISE(ABORT, 'insert on table "users" voilates foreign key users.owner_id')
  END;
END;

CREATE TRIGGER fku_users_owner_id
BEFORE UPDATE ON users
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((SELECT id FROM applications WHERE id = NEW.owner_id) IS NULL)
    THEN RAISE(ABORT, 'update on table "users" violates foreign key users.owner_id')
  END;
END;

CREATE TRIGGER fkd_users_owner_id
BEFORE DELETE ON applications
FOR EACH ROW BEGIN
  DELETE FROM users WHERE owner_id = OLD.id;
END;

-- users_sessions.user_id foreign key

CREATE TRIGGER fki_users_sessions_user_id
BEFORE INSERT ON users_sessions
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((SELECT id FROM users WHERE id = NEW.user_id) IS NULL)
    THEN RAISE(ABORT, 'insert on table "users_sessions" violates foreign key users_sessions.user_id')
  END;
END;

CREATE TRIGGER fku_users_sessions_user_id
BEFORE UPDATE ON users_sessions
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((SELECT id FROM users WHERE id = NEW.user_id) IS NULL)
    THEN RAISE(ABORT, 'update on table "users_sessions" violates foreign key users_sessions.user_id')
  END;
END;

CREATE TRIGGER fkd_users_sessions_user_id
BEFORE DELETE ON users
FOR EACH ROW BEGIN
  DELETE FROM users_sessions WHERE user_id = OLD.id;
END;

-- users_domain_sessions.session_id foreign key

CREATE TRIGGER fki_users_domain_sessions_session_id
BEFORE INSERT ON users_domain_sessions
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((SELECT id FROM users_sessions WHERE id = NEW.session_id) IS NULL)
    THEN RAISE(ABORT, 'insert on table "users_domain_sessions" violates foreign key users_sessions.session_id')
  END;
END;

CREATE TRIGGER fku_users_domain_sessions_session_id
BEFORE UPDATE ON users_domain_sessions
FOR EACH ROW BEGIN
  SELECT CASE
    WHEN ((SELECT id FROM users_sessions WHERE id = NEW.session_id) IS NULL)
    THEN RAISE(ABORT, 'update on table "users_domain_sessions" violates foreign key users_sessions.session_id')
  END;
END;

CREATE TRIGGER fkd_users_domain_sessions_session_id
BEFORE DELETE ON users_sessions
FOR EACH ROW BEGIN
  DELETE FROM users_domain_sessions WHERE session_id = OLD.id;
END;
